#include <mpi.h>
#include <stdlib.h>
#include <stdio.h>

static void mpi_err_handler(int error_code, MPI_Comm comm);

static const char datarep[] = "external32";

int main(void) {

  int exit_code = EXIT_SUCCESS;

  MPI_Init(NULL, NULL);

  int send_data[2] = {93184, -9023};
  int recv_data[2];

  MPI_Aint pack_size;
  mpi_err_handler(MPI_Pack_external_size(datarep, 2, MPI_INT, &pack_size),
                  MPI_COMM_WORLD);

  char * buffer = malloc(pack_size);
  MPI_Aint position = 0;

  mpi_err_handler(MPI_Pack_external(datarep, send_data, 2, MPI_INT, buffer,
                                    pack_size, &position), MPI_COMM_WORLD);
  position = 0;
  mpi_err_handler(MPI_Unpack_external(datarep, buffer, pack_size, &position,
                                      recv_data, 2, MPI_INT), MPI_COMM_WORLD);

  free(buffer);

  if ((send_data[0] != recv_data[0]) || (send_data[1] != recv_data[1]))
    exit_code = EXIT_FAILURE;

  MPI_Finalize();

  return exit_code;
}

//taken from http://beige.ucs.indiana.edu/I590/node85.html
static void mpi_err_handler(int error_code, MPI_Comm comm) {

   if (error_code != MPI_SUCCESS) {

      int rank;

      MPI_Comm_rank(comm, &rank);

      char error_string[1024];
      int length_of_error_string, error_class;

      MPI_Error_class(error_code, &error_class);
      MPI_Error_string(error_class, error_string, &length_of_error_string);
      fprintf(stderr, "%3d: %s\n", rank, error_string);
      MPI_Error_string(error_code, error_string, &length_of_error_string);
      fprintf(stderr, "%3d: %s\n", rank, error_string);
      MPI_Abort(comm, error_code);
   }
}

