package de.mpg.mpimet.yacgui.ui.dialog;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.mpg.mpimet.yacgui.controller.GuiController;
import de.mpg.mpimet.yacgui.ui.CouplingGui;
import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.InterpolationsTab;
import de.mpg.mpimet.yacgui.ui.dialog.tabpanel.TimestepTab;
import de.mpg.mpimet.yacgui.xml.TimestepUnit;
import de.mpg.mpimet.yacgui.xml.TransientCouple;
import de.mpg.mpimet.yacgui.junit.XmlDomHandlerTest;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

public class CouplingDialog extends JDialog {
	
	private static final long serialVersionUID = 5283512859321606370L;
	
	private static final String INTERPOLATION = "Interpolation";
	private static final String TIMESTEP = "Timestep";
	//private static final String MORE = "More";
	
	private static final String USE_AS_DEFAULT_LABEL = "Use these parameters as default";
	private static final String FORGET_DEFAULT_LABEL = "Forget default parameters";
	private static final String DEFAULT_TITLE = "Default parameters";
	private static final String USE_DEFAULT_LABEL = "Load default params";	
	
	private static final String LABEL_SAVE = "Save";
	private static final String LABEL_CLOSE = "Close";
	private static final int BUTTONS_VIEW_HEIGHT = 150;
	
	private InterpolationsTab interpolationsTab;
	private TimestepTab timestepTab;
	//private MoreTab moreTab;
	private TransientCouple transientCouple;
	private GuiController m_guiController;

	public CouplingDialog(String dialogCaption, String dialogDescription, TransientCouple transientCouple,
			TimestepUnit timestepUnit) {
		super((JDialog) null, true);
		
		this.transientCouple = transientCouple;
		
		setTitle(dialogCaption);
		setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		
		interpolationsTab = new InterpolationsTab(dialogDescription, transientCouple);
		timestepTab = new TimestepTab(dialogDescription, transientCouple, timestepUnit);
		//moreTab = new MoreTab(dialogDescription, transientCouple);
		
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab(INTERPOLATION, interpolationsTab);
		tabbedPane.addTab(TIMESTEP, timestepTab);
		//tabbedPane.addTab(MORE, moreTab);
		
		// init model and controller
		m_guiController = GuiController.getInstance();
		//m_guiController.registerCouplingGui(couplingGui);
		
		add(tabbedPane);
		add(createDefaultButtonsPanel());
		add(createButtonsPanel());
		pack();
	}
	
	private JPanel createDefaultButtonsPanel() {
		JPanel panel = new JPanel(new FlowLayout());

		//default button
		final JButton saveDefaultBut = new JButton();
		final JButton loadDefaultBut = new JButton(USE_DEFAULT_LABEL);

		if (!m_guiController.hasDefaultCouplingParams()) {
			saveDefaultBut.setText(USE_AS_DEFAULT_LABEL);
			loadDefaultBut.setEnabled(false);
		} else {
			saveDefaultBut.setText(FORGET_DEFAULT_LABEL);
		}
		
		saveDefaultBut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
				if (!m_guiController.hasDefaultCouplingParams()) {
					collectAllParams();
					m_guiController.setDefaultCouplingParams(transientCouple);
					
					saveDefaultBut.setText(FORGET_DEFAULT_LABEL);
					loadDefaultBut.setEnabled(true);
				} else {
					m_guiController.setDefaultCouplingParams(null);
					saveDefaultBut.setText(USE_AS_DEFAULT_LABEL);
					loadDefaultBut.setEnabled(false);
				}
			}
		});
		
		loadDefaultBut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TransientCouple defTransientCouple = m_guiController.getDefaultCouplingParams();
				
				if (defTransientCouple != null) {
					transientCouple.loadTransientCoupleData(defTransientCouple);
					
					interpolationsTab.loadParameters();
					timestepTab.loadParameters();
					//moreTab.loadParameters();
				}
			}
		});
		panel.add(saveDefaultBut);
		panel.add(loadDefaultBut);
		
		panel.setBorder(BorderFactory.createTitledBorder(DEFAULT_TITLE));
		panel.setMaximumSize(new Dimension(Integer.MAX_VALUE, BUTTONS_VIEW_HEIGHT));
		return panel;
	}
	
	protected JPanel createButtonsPanel() {
		JPanel bottom = new JPanel();
		final JButton saveButton = new JButton(LABEL_SAVE);
		saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
					collectAllParams();
					m_guiController.setTransientCoupleData(transientCouple);
					
					// added by Junpei Xu					
					JOptionPane.showMessageDialog(null, "transientCouple is saved");
			}
		});
		
		bottom.add(saveButton);
		final JButton closeButton = new JButton(LABEL_CLOSE);
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				CouplingDialog.this.dispose();
			}
		});
		bottom.add(closeButton);
		bottom.setMaximumSize(new Dimension(Integer.MAX_VALUE, BUTTONS_VIEW_HEIGHT));
		return bottom;
	}

	public void collectAllParams() {
		interpolationsTab.collectData();
		timestepTab.collectData();
		//moreTab.collectData();
	}
}
