/**
 * @file couple_config.c
 * @brief Structs and interfaces to defined coupling configuration
 *
 * Stores reads in coupling configuration and sets up data structures
 * to do the coupling.
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef COUPLE_CONFIG_H
#define COUPLE_CONFIG_H

#include "fields.h"
#include "component.h"
#include "yac_mpi.h"

/** \example test_couple_config_parallel1.c
 * This example show how to set up a couple configuration. It uses three
 * processes.
 */

/** \example test_couple_config_parallel2.c
 * This example show how to set up a couple configuration. It uses nine
 * processes.
 */

/** \example test_couple_config_parallel3.c
 * This example show how to set up a couple configuration. It uses nine
 * processes.
 */

/** \example test_couple_config_parallel4.c
 * This example show how to set up a couple configuration. It uses nine
 * processes.
 */

/**
 * reads coupling configuration from configuration xml file and initiates
 * the generation of all data structures required for interpolation
 * @param[in] num_components  number of locally defined components
 * @param[in] components      array of locally defined components
 * @param[in] num_fields      number of locally defined fields
 * @param[in] coupling_fields array of locally defined fields
 * \remarks this routine can only be called once
 */
void yac_setup_couple_config(unsigned num_components,
                             struct component * components[num_components],
                             unsigned num_fields,
                             struct coupling_field * coupling_fields[num_fields]);

/**
 * returns a communicator for communicating between pairs of components
 *
 * @param[in]  comp_name1     name of first component
 * @param[in]  comp_name2     name of second component
 */
MPI_Comm yac_get_pair_rootcomm_ ( char const * comp_name1,
                                  char const * comp_name2 );

#endif // COUPLE_CONFIG_H

