/**
 * @file interp_weights.h
 *
 * @copyright Copyright  (C)  2019 Moritz Hanke <hanke@dkrz.de>
 *                                 Thomas Jahns <jahns@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Thomas Jahns <jahns@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INTERP_WEIGHTS_H
#define INTERP_WEIGHTS_H

#include "core/core.h"
#include "interp_grid.h"
#include "interpolation.h"

enum interp_weights_reorder_type {
  MAPPING_ON_SRC, // weights will be appied at source processes
  MAPPING_ON_TGT, // weights will be applied at target processes
};

struct interp_weights;

struct interp_weights *
yac_interp_weights_new(struct interp_grid * interp_grid);

void yac_interp_weights_add_fixed(
  struct interp_weights * weights, struct remote_points * tgts,
  double fixed_value);

void yac_interp_weights_add_wsum(
  struct interp_weights * weights, struct remote_points * tgts,
  size_t * num_src_per_tgt, struct remote_point * srcs, double * w);

void yac_interp_weights_add_sum(
  struct interp_weights * weights, struct remote_points * tgts,
  size_t * num_src_per_tgt, struct remote_point * srcs);

void yac_interp_weights_add_direct(
  struct interp_weights * weights, struct remote_points * tgts,
  struct remote_point * srcs);

void yac_interp_weights_write_to_file(
  struct interp_weights * weights, char const * filename,
  char const * src_grid_name, char const * tgt_grid_name, int io_proc_step);

struct interpolation * yac_interp_weights_get_interpolation(
  struct interp_weights * weights, enum interp_weights_reorder_type reorder,
  size_t collection_size);

void yac_interp_weights_delete(struct interp_weights * weights);

#endif // INTERP_WEIGHTS_H
